# -*- coding: utf-8 -*-
from threading import Thread
import sched
from datetime import timedelta


def repeat(func, interval=timedelta(seconds=0), max_repeats=None):
    """
    Repeatedly invoke function at specified time interval

    Args:
        func: function to invoke

    Keyword args:
        interval (timedelta, default 0 seconds): interval to wait between each invocation
        max_repeats (integer, default None): maximum number of repeats

    Return values:
        cancel_fn (function): call this function to cancel repeat loop
    """
    cancelled = False

    def repeat(repeats_left=max_repeats or -1):
        if not cancelled and repeats_left != 0:
            try:
                func()
            finally:
                scheduler.enter(
                    interval.total_seconds(),
                    0,
                    repeat,
                    kwargs={'repeats_left': repeats_left - 1}
                )

    scheduler = sched.scheduler()
    scheduler.enter(0, 0, repeat)

    thread = Thread(
        target=lambda: scheduler.run(),
        daemon=True
    )
    thread.start()

    def cancel():
        nonlocal cancelled
        cancelled = True

    return cancel



import unittest
import time
from datetime import timedelta

from tinysched import scheduler


class TestSched(unittest.TestCase):
    def setUp(self):
        self.count = 0

    def test_repeat(self):
        def foo():
            self.count += 1

        repeats = 3
        scheduler.repeat(self.inc_counter, timedelta(seconds=0), max_repeats=repeats)
        time.sleep(0.1)
        self.assertEqual(self.count, repeats)

    def test_interval(self):
        repeats = 3
        delay = 0.01
        scheduler.repeat(self.inc_counter, timedelta(seconds=delay), max_repeats=repeats)
        for r in range(repeats):
            self.assertLessEqual(self.count, r + 1)
            time.sleep(delay)

    def test_cancel(self):
        delay = 0.01
        cancel_after_n = 10
        cancel = scheduler.repeat(self.inc_counter, timedelta(seconds=delay))
        time.sleep(delay * cancel_after_n)
        cancel()
        self.assertLessEqual(self.count, cancel_after_n)

    def inc_counter(self):
        self.count += 1



#!/usr/bin/env python

from setuptools import setup
from tinysched import __version__


setup(
    name='tinysched',
    version=__version__,
    description='Easily execute code repeatedly in background',
    author='Bryan Johnson',
    author_email='d.bryan.johnson@gmail.com',
    packages=['tinysched'],
    url='https://github.com/dbjohnson/tinysched',
    download_url='https://github.com/dbjohnson/tinysched/tarball/%s' % __version__
)
